package com.heibuddy.xiaohuoban.http;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.message.BasicHeader;

import android.text.TextUtils;

import com.heibuddy.xiaohuoban.util.CommonHelper;
import com.heibuddy.xiaohuoban.util.NetworkHelper;

public final class SimpleRequest {
    public static class Builder {
        private boolean post;
        private final List<SimpleRequestParam> params;
        private final List<Header> headers;
        private String url;

        public Builder() {
            this.post = false;
            this.params = new ArrayList<SimpleRequestParam>();
            this.headers = new ArrayList<Header>();
        }

        public SimpleRequest build() {
            return new SimpleRequest(this);
        }

        public Builder count(final int count) {
            this.params.add(new SimpleRequestParam("count", count));
            return this;
        }

        public Builder format(final String format) {
            if (!TextUtils.isEmpty(format)) {
                this.params.add(new SimpleRequestParam("format", format));
            }
            return this;
        }

        public Builder header(final Header header) {
            this.headers.add(header);
            return this;
        }

        public Builder header(final String name, final String value) {
            if (!TextUtils.isEmpty(name) && !TextUtils.isEmpty(value)) {
                this.headers.add(new BasicHeader(name, value));
            }
            return this;
        }

        public Builder headers(final List<Header> headers) {
            this.headers.addAll(headers);
            return this;
        }

        public Builder id(final String id) {
            if (!TextUtils.isEmpty(id)) {
                this.params.add(new SimpleRequestParam("id", id));
            }
            return this;
        }

        public Builder location(final String location) {
            if (!TextUtils.isEmpty(location)) {
                this.params.add(new SimpleRequestParam("location", location));
            }
            return this;
        }

        public Builder maxId(final String maxId) {
            if (!TextUtils.isEmpty(maxId)) {
                this.params.add(new SimpleRequestParam("max_id", maxId));
            }
            return this;
        }

        public Builder mode(final String mode) {
            if (!TextUtils.isEmpty(mode)) {
                this.params.add(new SimpleRequestParam("mode", mode));
            }
            return this;
        }

        public Builder page(final int page) {
            if (page > 0) {
                this.params.add(new SimpleRequestParam("page", page));
            }
            return this;
        }

        public Builder param(final NameValuePair pair) {
            this.params.add(new SimpleRequestParam(pair));
            return this;
        }

        public Builder param(final SimpleRequestParam param) {
            this.params.add(param);
            return this;
        }

        public Builder param(final String name, final boolean value) {
            if (!TextUtils.isEmpty(name)) {
                this.params.add(new SimpleRequestParam(name, value));
            }
            return this;
        }

        public Builder param(final String name, final File value) {
            if (!TextUtils.isEmpty(name) && (value != null)) {
                this.params.add(new SimpleRequestParam(name, value));
            }
            return this;
        }

        public Builder param(final String name, final int value) {
            if (!TextUtils.isEmpty(name)) {
                this.params.add(new SimpleRequestParam(name, value));
            }
            return this;
        }

        public Builder param(final String name, final long value) {
            if (!TextUtils.isEmpty(name)) {
                this.params.add(new SimpleRequestParam(name, value));
            }
            return this;
        }

        public Builder param(final String name, final String value) {
            if (!TextUtils.isEmpty(name) && !TextUtils.isEmpty(value)) {
                this.params.add(new SimpleRequestParam(name, value));
            }
            return this;
        }

        public Builder params(final List<SimpleRequestParam> params) {
            this.params.addAll(params);
            return this;
        }

        public Builder post() {
            this.post = true;
            return this;
        }

        public Builder post(final boolean post) {
            this.post = post;
            return this;
        }

        public Builder sinceId(final String sinceId) {
            if (!TextUtils.isEmpty(sinceId)) {
                this.params.add(new SimpleRequestParam("since_id", sinceId));
            }
            return this;
        }

        public Builder status(final String status) {
            if (!TextUtils.isEmpty(status)) {
                this.params.add(new SimpleRequestParam("status", status));
            }
            return this;
        }

        public Builder url(final String url) {
            if (TextUtils.isEmpty(url)) {
                throw new IllegalArgumentException(
                        "Builder.url() request url must not be empty or null.");
            }
            this.url = url;
            return this;
        }

    }

    private static final String TAG = SimpleRequest.class.getSimpleName();

    public static Builder newBuilder() {
        return new Builder();
    }

    public final boolean post;
    private final List<SimpleRequestParam> params;
    private final List<Header> headers;
    public final HttpEntity entity;

    public final String url;

    private final HttpRequestBase httpRequest;

    private SimpleRequest(final Builder builder) {
        this.post = builder.post;
        this.headers = builder.headers;
        this.params = builder.params;
        if (this.post) {
            this.url = builder.url;
            this.httpRequest = new HttpPost(this.url);
            if (!CommonHelper.isEmpty(this.params)) {
                if (SimpleRequestParam.hasFile(this.params)) {
                    this.entity = NetworkHelper
                            .encodeMultipartParameters(this.params);
                } else {
                    this.entity = NetworkHelper
                            .encodePostParameters(this.params);
                }
                ((HttpPost) this.httpRequest).setEntity(this.entity);
            } else {
                this.entity = null;
            }
        } else {
            this.entity = null;
            if (CommonHelper.isEmpty(this.params)) {
                this.url = builder.url;
            } else {
                this.url = builder.url + "?"
                        + NetworkHelper.encodeQueryParameters(this.params);
                ;
            }
            this.httpRequest = new HttpGet(this.url);
        }

        if (this.headers != null) {
            for (final Header header : this.headers) {
                this.httpRequest.addHeader(header);
            }
        }
    }

    public void abort() {
        if (this.httpRequest != null) {
            this.httpRequest.abort();
        }
    }

    public List<Header> getHeaders() {
        return this.headers;
    }

    public HttpRequestBase getHttpRequest() {
        return this.httpRequest;
    }

    public List<SimpleRequestParam> getParams() {
        return this.params;
    }

}
